﻿using System.Collections.Generic;
using RimWorld;
using RimWorld.Planet;
using VanillaPsycastsExpanded;
using Verse;
using Verse.AI;
using Verse.Sound;
using Ability = VFECore.Abilities.Ability;

namespace Stunskip
{
    public class Stunskip_Strike_ii : Ability
    {
        private int attackInTicks = -1;

        public override void Cast(params GlobalTargetInfo[] targets)
        {
            base.Cast(targets);
            AttackTarget((LocalTargetInfo)targets[0]);
        }

        public override void Tick()
        {
            base.Tick();
            if (attackInTicks != -1 && Find.TickManager.TicksGame >= attackInTicks)
            {
                attackInTicks = -1;
                var target = FindAttackTarget();
                if (target != null) AttackTarget(target);
            }
        }

        private void AttackTarget(LocalTargetInfo target)
        {

            pawn.Position = target.Cell;
            pawn.Notify_Teleported(false);
            pawn.stances.SetStance(new Stance_Mobile());
            VerbProperties_AdjustedMeleeDamageAmount_Patch.multiplyByPawnMeleeSkill = false;
            pawn.meleeVerbs.TryMeleeAttack(target.Pawn, null, true);
            pawn.meleeVerbs.TryMeleeAttack(target.Pawn, null, true);
            VerbProperties_AdjustedMeleeDamageAmount_Patch.multiplyByPawnMeleeSkill = false;

        }

        private Pawn FindAttackTarget()
        {
            var targetScanFlags = TargetScanFlags.NeedLOSToPawns | TargetScanFlags.NeedReachableIfCantHitFromMyPos | TargetScanFlags.NeedThreat
                                  | TargetScanFlags.NeedAutoTargetable;
            return (Pawn)AttackTargetFinder.BestAttackTarget(pawn, targetScanFlags, x => x is Pawn pawn && !pawn.Dead, 0f, 999999);
        }

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref attackInTicks, "attackInTicks", -1);
        }
    }
}